@@ -1,4 +1,4 @@
-# Gufw 10.04.5 - http://gufw.tuxfamily.org
+# Gufw 10.10.1 - http://gufw.tuxfamily.org
 # Copyright (C) 2008-2010 Marcos Alvarez Costales <marcos@ubuntu-ast.org>
 #
 # Gufw is free software; you can redistribute it and/or modify
@@ -28,10 +28,18 @@ class Firewall:
         self.rule = Rule()
         
         self.status = self.get_ufw_status()
+        self.report = self.get_initial_report()
         self.incoming_policy = self.get_ufw_policy("incoming")
         self.outgoing_policy = self.get_ufw_policy("outgoing")
         self.ufw_logging = self.get_ufw_log()
-        
+    
+    def get_initial_report(self):
+        cmd = commands.getstatusoutput("grep report=enable /etc/gufw/gufw.cfg")
+        if cmd[1].find("enable") != -1:
+            return "enable"
+        else:
+            return "disable"
+    
     def get_ufw_status(self):
         ufw_status = commands.getstatusoutput("LANGUAGE=C ufw status")
         if ufw_status[1].find("Status: active") != -1:
@@ -75,6 +83,14 @@ class Firewall:
         """Return actual status"""
         return self.status
     
+    def get_report(self):
+        """Return actual listening report"""
+        return self.report
+    
+    def set_report(self,report):
+        """Set actual listening report"""
+        self.report = report
+        
     def set_status(self, status):
         """Set status to ufw [enable|disable]"""
         if status == "enable":
@@ -87,6 +103,53 @@ class Firewall:
         ufw_cmd = commands.getstatusoutput(cmd)
         self.gufw_log.add_log(cmd)
     
+    def get_listening_report(self):
+        return_report = []
+        actual_protocol = "None"
+        
+        ufw_report = commands.getstatusoutput("LANGUAGE=C ufw show listening")
+        report_lines = ufw_report[1].replace("\n   [","%")
+        report_lines = report_lines.split("\n")
+        for descomponent_report in report_lines:
+            # Set actual protocol
+            if descomponent_report == "":
+                continue
+            if descomponent_report.find("tcp6:") != -1:
+                actual_protocol = "TCP6"
+                continue
+            if descomponent_report.find("tcp:") != -1:
+                actual_protocol = "TCP"
+                continue
+            if descomponent_report.find("udp6:") != -1:
+                actual_protocol = "UDP6"
+                continue
+            if descomponent_report.find("udp:") != -1:
+                actual_protocol = "UDP"
+                continue
+                
+            policy = "None"
+            descomponent_report = descomponent_report.strip()
+            descomponent_report = descomponent_report.replace("(","")
+            descomponent_report = descomponent_report.replace(")","")
+            
+            if descomponent_report.find("]") != -1:
+                descomponent_policy = descomponent_report.split("]")
+                if descomponent_policy[1].find("allow") != -1:
+                    policy = "allow"
+                elif descomponent_policy[1].find("deny") != -1:
+                    policy = "deny"
+                elif descomponent_policy[1].find("reject") != -1:
+                    policy = "reject"
+                elif descomponent_policy[1].find("limit") != -1:
+                    policy = "limit"
+            
+            descomponent_report = descomponent_report.split("%")
+            descomponent_fields = descomponent_report[0].split(" ")
+            # Order: protocol % port % address % application % policy
+            return_report.append(actual_protocol + "%" + descomponent_fields[0] + "%" + descomponent_fields[1] + "%" + descomponent_fields[2] + "%" + policy)
+        
+        return return_report
+    
     def get_default_policy(self, policy):
         """Get actual policy for incoming or outgoing traffic [allow|deny|reject]"""
         if policy == "incoming":
