@@ -1,4 +1,4 @@
-# Gufw 10.04.5 - http://gufw.tuxfamily.org
+# Gufw 10.10.1 - http://gufw.tuxfamily.org
 # Copyright (C) 2008-2010 Marcos Alvarez Costales <marcos@ubuntu-ast.org>
 #
 # Gufw is free software; you can redistribute it and/or modify
@@ -22,9 +22,9 @@ import commands
 class GufwLog:
     """Record and return Gufw Log"""
     def __init__(self):
-        self.logging = self.get_ufw_logging()
+        self.logging = self.get_gufw_logging()
     
-    def get_ufw_logging(self):
+    def get_gufw_logging(self):
         cmd = commands.getstatusoutput("grep log=enable /etc/gufw/gufw.cfg")
         if cmd[1].find("enable") != -1:
             return "enable"
