@@ -1,4 +1,4 @@
-# Gufw 10.04.5 - http://gufw.tuxfamily.org
+# Gufw 10.10.1 - http://gufw.tuxfamily.org
 # Copyright (C) 2008-2010 Marcos Alvarez Costales <marcos@ubuntu-ast.org>
 #
 # Gufw is free software; you can redistribute it and/or modify
@@ -22,6 +22,7 @@ import sys
 import locale
 import gettext
 import commands
+import threading
 
 # Locale translations
 DIR = "/usr/share/locale-langpack"
@@ -36,11 +37,13 @@ class GuiGufw:
     """All events of glade file"""
     def __init__(self, firewall):
         self.firewall = firewall
-        
+
         # Set GUI objects
         self.glade  = gtk.glade.XML(sys.path[0] + "/glade/gufw.glade")
-        self.model  = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING,
-                                    gobject.TYPE_STRING, gobject.TYPE_STRING) 
+        self.model_rules  = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING,
+                                          gobject.TYPE_STRING, gobject.TYPE_STRING) 
+        self.model_report = gtk.ListStore(gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING,
+                                          gobject.TYPE_STRING, gobject.TYPE_STRING, gobject.TYPE_STRING) 
         self.render = gtk.CellRendererText()
         
         self.set_glade_objects()
@@ -60,18 +63,19 @@ class GuiGufw:
         self.cb_policy_incoming    = self.glade.get_widget("cbPolicyIncoming")
         self.cb_policy_outgoing    = self.glade.get_widget("cbPolicyOutgoing")
         self.shield                = self.glade.get_widget("imgShield")
+        self.blockReport           = self.glade.get_widget("blockReport")
         self.btn_add_window        = self.glade.get_widget("btnAddWindow")
         self.btn_remove_rule       = self.glade.get_widget("btnRemove")
         self.status_bar            = self.glade.get_widget("statusBar")
         self.progress_bar          = self.glade.get_widget("progressBar")
         self.separator_progressbar = self.glade.get_widget("separator_progressbar")
         
-        # Rules
-        self.tv_rules = self.glade.get_widget("tvRules")
-        self.tv_rules.set_model(self.model)
-        self.tv_rules.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
         # Font
         self.render.set_property("font", "monospace 10")
+        # Rules
+        self.tv_rules = self.glade.get_widget("tvRules")
+        self.tv_rules.set_model(self.model_rules)
+        self.tv_rules.get_selection().set_mode(gtk.SELECTION_MULTIPLE)
         # The format columns
         tree_header = gtk.TreeViewColumn (_("To"), self.render, text=1, foreground=4)
         tree_header.set_resizable(True)
@@ -82,6 +86,23 @@ class GuiGufw:
         tree_header = gtk.TreeViewColumn (_("From"), self.render, text=3, foreground=4)
         self.tv_rules.append_column (tree_header)
         
+        # Listening Report
+        self.tv_report = self.glade.get_widget("tvReport")
+        self.tv_report.set_model(self.model_report)
+        self.tv_report.get_selection().set_mode(gtk.SELECTION_NONE)
+        # The format columns
+        tree_header = gtk.TreeViewColumn (_("Protocol"), self.render, text=1, foreground=5)
+        tree_header.set_resizable(True)
+        self.tv_report.append_column (tree_header)
+        tree_header = gtk.TreeViewColumn (_("Port"), self.render, text=2, foreground=5)
+        tree_header.set_resizable(True)
+        self.tv_report.append_column (tree_header)
+        tree_header = gtk.TreeViewColumn (_("Address"), self.render, text=3, foreground=5)
+        tree_header.set_resizable(True)
+        self.tv_report.append_column (tree_header)
+        tree_header = gtk.TreeViewColumn (_("Application"), self.render, text=4, foreground=5)
+        self.tv_report.append_column (tree_header)
+        
         # Preconf
         self.extended_actions          = self.glade.get_widget("cbExtendedActions")
         self.insert_number_preconf     = self.glade.get_widget("sbInsertNumberPreconf")
@@ -129,6 +150,7 @@ class GuiGufw:
         self.server_script  = self.glade.get_widget("cbServerScript")
         # Preference Window
         self.winpreferences = self.glade.get_widget("winPreferences")
+        self.cb_report      = self.glade.get_widget("cbReport")
         self.cb_ufw_log     = self.glade.get_widget("cbUfwLog")
         self.lbl_ufw_level  = self.glade.get_widget("lblLogLevel")
         self.cb_ufw_level   = self.glade.get_widget("cbLogLevel")
@@ -165,6 +187,14 @@ class GuiGufw:
         # Focus
         self.btn_add_window.grab_focus()
         
+        if self.firewall.get_report() == "enable":
+            self.blockReport.show()
+            self.start_refresh = "enable"
+            self.refresh_report()
+        else:
+            self.blockReport.hide()
+            self.start_refresh = "disable"
+        
     def set_old_size_window(self):
         """Save Actual size window"""
         width_height = commands.getstatusoutput("grep sizewin /etc/gufw/gufw.cfg")
@@ -174,7 +204,7 @@ class GuiGufw:
             height = int(width_height_split[1])
         else:
             width  = 336
-            height = 404
+            height = 334
             
         screen_width  = gtk.gdk.screen_width()
         screen_height = gtk.gdk.screen_height()
@@ -190,7 +220,13 @@ class GuiGufw:
         self.cb_enabled_firewall.set_sensitive(1)
         if self.firewall.get_status() == "enable":
             self.cb_enabled_firewall.set_active(1)
-
+        
+        # Listening report
+        if self.firewall.get_report() == "enable":
+            self.blockReport.show()
+        else:
+            self.blockReport.hide()
+        
         # Allow/Deny/Reject Incoming
         if self.firewall.get_default_policy("incoming") == "deny":
             self.cb_policy_incoming.set_active(0)
@@ -248,7 +284,7 @@ class GuiGufw:
         
     def set_text_rules(self):
         """Show rules in GUI"""
-        self.model.clear() 
+        self.model_rules.clear() 
         
         if self.firewall.get_status() == "disable":
             return
@@ -264,94 +300,94 @@ class GuiGufw:
         rules = self.firewall.get_rules()
         for rule in rules:
             row += 1
-            iterador = self.model.insert(row)
+            iterador = self.model_rules.insert(row)
             
             # Color Allow/Deny/Reject/Limit
             # IN mode (equal to normal mode, persist code for clear read)
             if rule.find("ALLOW IN") != -1:
                 split_str = "ALLOW IN"
                 if self.firewall.get_default_policy("incoming") != "allow":
-                    self.model.set_value(iterador, 4, color["red"])
+                    self.model_rules.set_value(iterador, 4, color["red"])
                 else:
-                    self.model.set_value(iterador, 4, color["gray"])
+                    self.model_rules.set_value(iterador, 4, color["gray"])
             # Deny?
             elif rule.find("DENY IN") != -1:
                 split_str = "DENY IN"
                 if self.firewall.get_default_policy("incoming") != "deny":
-                    self.model.set_value(iterador, 4, color["green"])
+                    self.model_rules.set_value(iterador, 4, color["green"])
                 else:
-                    self.model.set_value(iterador, 4, color["gray"])
+                    self.model_rules.set_value(iterador, 4, color["gray"])
             # Reject
             elif rule.find("REJECT IN") != -1:
                 split_str = "REJECT IN"
                 if self.firewall.get_default_policy("incoming") != "reject":
-                    self.model.set_value(iterador, 4, color["blue"])
+                    self.model_rules.set_value(iterador, 4, color["blue"])
                 else:
-                    self.model.set_value(iterador, 4, color["gray"])
+                    self.model_rules.set_value(iterador, 4, color["gray"])
             # Limit?
             elif rule.find("LIMIT IN") != -1:
                 split_str = "LIMIT IN"
-                self.model.set_value(iterador, 4, color["orange"])
+                self.model_rules.set_value(iterador, 4, color["orange"])
             
             # OUT mode
             elif rule.find("ALLOW OUT") != -1: 
                 split_str = "ALLOW OUT"
                 if self.firewall.get_default_policy("outgoing") != "allow":
-                    self.model.set_value(iterador, 4, color["red"]) 
+                    self.model_rules.set_value(iterador, 4, color["red"]) 
                 else:
-                    self.model.set_value(iterador, 4, color["gray"])
+                    self.model_rules.set_value(iterador, 4, color["gray"])
             # Deny?
             elif rule.find("DENY OUT") != -1:
                 split_str = "DENY OUT"
                 if self.firewall.get_default_policy("outgoing") != "deny":
-                    self.model.set_value(iterador, 4, color["green"])
+                    self.model_rules.set_value(iterador, 4, color["green"])
                 else:
-                    self.model.set_value(iterador, 4, color["gray"])
+                    self.model_rules.set_value(iterador, 4, color["gray"])
             # Reject
             elif rule.find("REJECT OUT") != -1:
                 split_str = "REJECT OUT"
                 if self.firewall.get_default_policy("outgoing") != "reject":
-                    self.model.set_value(iterador, 4, color["blue"])
+                    self.model_rules.set_value(iterador, 4, color["blue"])
                 else:
-                    self.model.set_value(iterador, 4, color["gray"])
+                    self.model_rules.set_value(iterador, 4, color["gray"])
             # Limit?
             elif rule.find("LIMIT OUT") != -1:
                 split_str = "LIMIT OUT"
-                self.model.set_value(iterador, 4, color["orange"])
+                self.model_rules.set_value(iterador, 4, color["orange"])
                 
             # NORMAL mode
             # Allow?
             elif rule.find("ALLOW") != -1: 
                 split_str = "ALLOW"
                 if self.firewall.get_default_policy("incoming") != "allow":
-                    self.model.set_value(iterador, 4, color["red"]) 
+                    self.model_rules.set_value(iterador, 4, color["red"]) 
                 else:
-                    self.model.set_value(iterador, 4, color["gray"])
+                    self.model_rules.set_value(iterador, 4, color["gray"])
             # Deny?
             elif rule.find("DENY") != -1:
                 split_str = "DENY"
                 if self.firewall.get_default_policy("incoming") != "deny":
-                    self.model.set_value(iterador, 4, color["green"])
+                    self.model_rules.set_value(iterador, 4, color["green"])
                 else:
-                    self.model.set_value(iterador, 4, color["gray"])
+                    self.model_rules.set_value(iterador, 4, color["gray"])
             # Reject
             elif rule.find("REJECT") != -1:
                 split_str = "REJECT"
                 if self.firewall.get_default_policy("incoming") != "reject":
-                    self.model.set_value(iterador, 4, color["blue"])
+                    self.model_rules.set_value(iterador, 4, color["blue"])
                 else:
-                    self.model.set_value(iterador, 4, color["gray"])
+                    self.model_rules.set_value(iterador, 4, color["gray"])
             # Limit?
             elif rule.find("LIMIT") != -1:
                 split_str = "LIMIT"
-                self.model.set_value(iterador, 4, color["orange"])
+                self.model_rules.set_value(iterador, 4, color["orange"])
             
             # Values
             rule_split = rule.split(split_str)
-            self.model.set_value(iterador, 0, row) # Row (use for remove rule)
-            self.model.set_value(iterador, 1, rule_split[0].strip())
-            self.model.set_value(iterador, 2, split_str)
-            self.model.set_value(iterador, 3, rule_split[1].strip())
+            self.model_rules.set_value(iterador, 0, row) # Row (use for remove rule)
+            self.model_rules.set_value(iterador, 1, rule_split[0].strip())
+            self.model_rules.set_value(iterador, 2, split_str)
+            self.model_rules.set_value(iterador, 3, rule_split[1].strip())
             
     # Save Actual size window
     def save_actual_window_size(self):
@@ -364,6 +400,16 @@ class GuiGufw:
         cmd = cmd.replace("new_size_window", new_size)
         commands.getstatusoutput(cmd)
 
+    # Save Actual listening report
+    def save_actual_report(self):
+        """Save actual listening report for next instances"""
+        if self.firewall.get_report() == "enable":
+            cmd = "sed -i 's/report.*$/report=enable/' /etc/gufw/gufw.cfg"
+            commands.getstatusoutput(cmd)
+        else:
+            cmd = "sed -i 's/report.*$/report=disable/' /etc/gufw/gufw.cfg"
+            commands.getstatusoutput(cmd)
+
 
 
 
@@ -375,11 +421,13 @@ class GuiGufw:
     # Close Windows
     def winMain_delete_event_cb(self, widget, event):
         self.save_actual_window_size()
+        self.save_actual_report()
         gtk.main_quit()
 
     # Menu Quit
     def menuQuit_activate_cb(self, widget):
         self.save_actual_window_size()
+        self.save_actual_report()
         gtk.main_quit()
     
     # Button Add
@@ -460,6 +508,11 @@ class GuiGufw:
     # Show Preferences
     def menuPreferences_activate_cb(self, widget):
         # Set initial values
+        if self.firewall.get_report() == "enable":
+            self.cb_report.set_active(1)
+        else:
+            self.cb_report.set_active(0)
+        
         if self.firewall.get_status() == "disable":
             self.cb_ufw_log.set_active(0)
             self.cb_ufw_log.set_sensitive(0)
@@ -574,8 +627,21 @@ class GuiGufw:
             self.firewall.set_gufw_logging("enable")
         elif self.firewall.get_gufw_logging() != "disable" and self.cb_gufw_log.get_active() == 0:
             self.firewall.set_gufw_logging("disable")
-            
         self.set_window_values("")
+    
+    # Listening report
+    def cbReport_toggled_cb(self, widget):
+        if self.firewall.get_report() == "disable" and self.cb_report.get_active() == 1:
+            self.firewall.set_report("enable")
+            self.blockReport.show()
+            self.start_refresh = "enable"
+        
+        elif self.firewall.get_report() == "enable" and self.cb_report.get_active() == 0:
+            self.firewall.set_report("disable")
+            self.blockReport.hide()
+            self.start_refresh = "disable"
+        
+        self.refresh_report()
 
 
 
@@ -677,8 +743,21 @@ class GuiGufw:
         # Remove in 2 plan
         task = self.remove_rules()
         gobject.idle_add(task.next)
-        
+            
+    # Refresh method in background (no freeze)
+    def do_refresh_report(self):
+        listening_lines = self.firewall.get_listening_report()
+        background_job = RefreshReport(self.firewall.get_status(),self.model_report, listening_lines)
+        background_job.start()
+        if self.start_refresh == "enable":
+            return True
+        else:
+            return False
     
+    # Button refresh Listening report
+    def refresh_report(self):
+        gobject.timeout_add(1500, self.do_refresh_report)
+        
     # Remove Rules Method
     def remove_rules(self):
         tree,iter  = self.tv_rules.get_selection().get_selected_rows()
@@ -763,7 +842,7 @@ class GuiGufw:
                      "KTorrent"     : "6881tcp#4444udp",
                      "Nicotine"     : "2234:2239tcp#2242tcp",
                      "qBittorrent"  : "6881tcp",
-                     "Transmission" : "51413tcp" }
+                     "Transmission" : "51413tcp#51413udp" }
         # Define Services 
         services = { "FTP"         : "ftp",
                      "HTTP"        : "http",
@@ -919,10 +998,10 @@ class GuiGufw:
         port = self.port_simple.get_text()
         
         # ? -> !
-        if port == "bulls":
-            dlg_egg = gtk.MessageDialog(parent=self.win_main, flags=0, type=gtk.MESSAGE_WARNING, buttons=gtk.BUTTONS_CLOSE, message_format="No bullfighting in Spain!")
-            dlg_egg.format_secondary_markup("http://acurti.es/wxa")
-            dlg_egg.set_title("SOS - Toros NON!")
+        if port == "freedom":
+            dlg_egg = gtk.MessageDialog(parent=self.win_main, flags=0, type=gtk.MESSAGE_WARNING, buttons=gtk.BUTTONS_CLOSE, message_format="I could have made money this way, and perhaps amused myself writing code. But I knew that at the end of my career, I would look back on years of building walls to divide people, and feel I had spent my life making the world a worse place.")
+            dlg_egg.format_secondary_markup("Richard Stallman")
+            dlg_egg.set_title("It's time to think!")
             dlg_egg.run()
             dlg_egg.destroy()
             return
@@ -1053,3 +1132,51 @@ class GuiGufw:
             self.box_log_simple.show()
             self.box_log_advanced.show()
 
+
+
+
+
+
+
+
+
+class RefreshReport(threading.Thread):
+    """Refresh Listening report in background"""
+    def __init__(self, firewall_status, model, lines):
+        threading.Thread.__init__(self)
+        self.firewall_status = firewall_status
+        self.model_report = model
+        self.listening = lines
+    
+    def run(self):
+        """Show listening report in GUI"""
+        self.model_report.clear() 
+        
+        row = 0
+        # GNOME Palette
+        color = { "green"  : "#267726",  # Accent Green Dark
+                  "red"    : "#DF421E",  # Accent Red
+                  "orange" : "#D1940C",  # Accent Yellow Dark
+                  "blue"   : "#314E6C" } # Blue Shadow
+        
+        for report in self.listening:
+            row += 1
+            iterador = self.model_report.insert(row)
+            report_split = report.split("%")
+            
+            self.model_report.set_value(iterador, 0, row)
+            self.model_report.set_value(iterador, 1, report_split[0].strip()) # Protocol
+            self.model_report.set_value(iterador, 2, report_split[1].strip()) # Port
+            self.model_report.set_value(iterador, 3, report_split[2].strip()) # Address
+            self.model_report.set_value(iterador, 4, report_split[3].strip()) # App
+            # Only apply colors when firewall rules apply to that (fw = enabled)
+            if self.firewall_status == "enable":
+                if report_split[4] == "allow":
+                    self.model_report.set_value(iterador, 5, color["red"])
+                elif report_split[4] == "deny":
+                    self.model_report.set_value(iterador, 5, color["green"])
+                elif report_split[4] == "reject":
+                    self.model_report.set_value(iterador, 5, color["blue"])
+                elif report_split[4] == "limit":
+                    self.model_report.set_value(iterador, 5, color["orange"])
+
